/**
 * Copyright (C) 2009, Progress Software Corporation and/or its 
 * subsidiaries or affiliates.  All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
body, html {
    font-family:georgia;
    color: #666666;
}
body, html, table, tr, td, tbody {
    padding:0;
    margin:0;
    vertical-align: top;
}

h1, h2, h3, h4, h5, h6 {
    overflow: visible;
    color: black;
}
p { margin-bottom:0px; }

a { text-decoration: none; color:#00AA00; }
a:hover { color: #820000; text-decoration: underline;}
img {border: none;}

a.main { font-size: 12pt; }

.wrapper { min-width: 762px; width: 762px; margin: 0 auto; }

#navigation { 
    width: 100%; clear: both;
    font-family: HelveticaNeue,"Helvetica Neue",Helvetica,Arial,Verdana,sans-serif; 
    background: #0F0F0F; text-align:center; padding: 8px;
    border-bottom:2px solid #00FF00;
    color: white; font-size: 10pt; font-weight: bold;
}
#navigation a { color: white; }
#navigation ul { display: block; margin:0; padding:0; }
#navigation li { list-style-type: none; display: inline; margin:0 5px 0 5px;}

#overview { width: 100%; clear: both; padding: 0px 0px 20px 0px;}
#overview div.logo {padding: 0px 20px 20px 0px; float: left;}
#overview div.message { 
    font-size: 14pt; 
}
#overview div.message h1{ margin-bottom: 5px; }

#spot { width: 100%; 
    margin-top: 15px;
    background:transparent url(../images/spot-banner.gif) repeat-x scroll 0 0;
    height: 277px;
}
#spot div.title {
    text-align:center; height:25px;
    text-align:center; padding-top: 2px;
    color: white; font-size: 10pt; font-weight: bold; 
    font-family: HelveticaNeue,"Helvetica Neue",Helvetica,Arial,Verdana,sans-serif; 
}
#spot div.content table {
    width: 100%; text-align:center; 
    color: black; font-size: 10pt; 
}

#spot a:hover {text-decoration: none; }

#spot div.spot-balloon {
  background:transparent url( ../../../images/spot-get-involved.gif ) no-repeat;
  background-position:center;
  height: 121px;
  text-align:left;
  padding-top: 25px;
  padding-left: 35px;
}

#spot div.spot-balloon li {
    
  list-style: none;
}

#content { width: 100%; clear: both; }
#content .wrapper { min-width: 700px; width: 700px; margin: 0 auto; }
#content div.left {
  float:left;
  width:200px;
  text-align:right;
  font-size: 18pt;
  clear:both;
}
#content h1 {
  font-size: 18pt;
}
#content div.right {
  float:right;
  width:450px;
  text-align:left;
}

#blog { width: 100%;  }
#blog .wrapper { min-width: 600px; width: 600px; margin: 0 auto; }
#blog h1 {
  font-family: HelveticaNeue,"Helvetica Neue",Helvetica,Arial,Verdana,sans-serif; 
  font-size: 18pt; color: #00AA00;
}
#blog h2 {
  border-bottom: thin dashed #DDD;
  font-size: 16pt;
  margin-bottom: 5px;
}
#blog div.post p {
  padding-left: 10px;
}
#blog div.post .details {
  padding-top: 5px;
  color: #ccc;
  font-size: 10pt;
  font-family: HelveticaNeue,"Helvetica Neue",Helvetica,Arial,Verdana,sans-serif; 
}

#content .post h2 {
  margin-bottom:5px;
}
#content .post .details {
  color: #ccc;
  font-size: 10pt;
  font-family: HelveticaNeue,"Helvetica Neue",Helvetica,Arial,Verdana,sans-serif; 
  margin-top:0px;
}

img.example {
  padding: 10px;
}